package com.muyu.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.muyu.common.core.annotation.Excel;
import com.muyu.domain.req.FaultRecordEditReq;
import com.muyu.domain.req.FaultRecordQueryReq;
import com.muyu.domain.req.FaultRecordSaveReq;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.Date;

/** 故障记录
 * @ClassDescription:
 * @JdkVersion: 17
 * @Author: zhangxu
 * @Created: 2024/6/20 15:17
 */

@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("fault_record")
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "FaultRecord", description = "故障记录")
public class FaultRecord {
    private static final long serialVersionUID = 1L;

    /** 主键 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "主键", value = "主键")
    private Long id;

    /** 故障码 */
    @Excel(name = "故障码")
    @ApiModelProperty(name = "故障码", value = "故障码")
    private String faultCode;

    /** 车辆vin */
    @Excel(name = "车辆vin")
    @ApiModelProperty(name = "车辆vin", value = "车辆vin")
    private String vin;

    /** 故障开始时间 */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "故障开始时间", width = 30, dateFormat = "yyyy-MM-dd")
    @ApiModelProperty(name = "故障开始时间", value = "故障开始时间")
    private Date startTime;

    /** 故障结束时间 */
    @JsonFormat(pattern = "yyyy-MM-dd")
    @Excel(name = "故障结束时间", width = 30, dateFormat = "yyyy-MM-dd")
    @ApiModelProperty(name = "故障结束时间", value = "故障结束时间")
    private Date endTime;

    /** 故障级别 */
    @Excel(name = "故障级别")
    @ApiModelProperty(name = "故障级别", value = "故障级别")
    private String faultLevel;

    /** 故障是否处理 */
    @Excel(name = "故障是否处理")
    @ApiModelProperty(name = "故障是否处理", value = "故障是否处理")
    private String faultHandle;

    /**
     * 查询构造器
     */
    public static FaultRecord queryBuild(FaultRecordQueryReq faultRecordQueryReq){
        return FaultRecord.builder()
                .faultCode(faultRecordQueryReq.getFaultCode())
                .vin(faultRecordQueryReq.getVin())
                .build();
    }

    /**
     * 添加构造器
     */
    public static FaultRecord saveBuild(FaultRecordSaveReq faultRecordSaveReq){
        return FaultRecord.builder()
                .faultCode(faultRecordSaveReq.getFaultCode())
                .vin(faultRecordSaveReq.getVin())
                .startTime(faultRecordSaveReq.getStartTime())
                .endTime(faultRecordSaveReq.getEndTime())
                .faultLevel(faultRecordSaveReq.getFaultLevel())
                .faultHandle(faultRecordSaveReq.getFaultHandle())
                .build();
    }

    /**
     * 修改构造器
     */
    public static FaultRecord editBuild(Long id, FaultRecordEditReq faultRecordEditReq){
        return FaultRecord.builder()
                .id(id)
                .faultCode(faultRecordEditReq.getFaultCode())
                .vin(faultRecordEditReq.getVin())
                .startTime(faultRecordEditReq.getStartTime())
                .endTime(faultRecordEditReq.getEndTime())
                .faultLevel(faultRecordEditReq.getFaultLevel())
                .faultHandle(faultRecordEditReq.getFaultHandle())
                .build();
    }

}
